@@ -103,10 +103,10 @@ class SaveBest(Callback):
         if self.model.fit_params.state != 'valid': return
         losses, metric = self.metric_log.val_epoch_results
         if metric is None or len(losses) > 1:  # Tracking SWA only supported for loss
-            val,mi = losses.min(),losses.argmin()
+            val,mi = np.nanmin(losses),np.nanargmin(losses)
         else:
             val,mi = metric,0
-        if val < self.min_val:
+        if val < self.min_val or np.isnan(self.min_val) or self.min_val is math.inf:
             self.min_val = val
             m = self.model
             if mi > 0: m = self.model.fit_params.loss_cbs[mi-1].test_model
@@ -198,7 +198,7 @@ class MetricLogger(Callback):
             if not self.metric_cbs[self.main_metric_idx].lower_metric_better: m *= -1
         self.val_epoch_results = ls,m
 
-    def on_forwards_end(self) -> None:
+    def on_batch_end(self) -> None:
         r'''
         Record batch loss
         '''
@@ -270,13 +270,14 @@ class MetricLogger(Callback):
                     self.epochs = self.epochs[1:]
                     for i in range(len(self.vel_vals)): self.vel_vals[i],self.gen_vals[i] = self.vel_vals[i][1:],self.gen_vals[i][1:]
 
+            # Metrics
             if self.main_metric_idx is not None:
                 self.metric_ax.clear()
                 with sns.axes_style(**self.plot_settings.style), sns.color_palette(self.plot_settings.cat_palette) as palette:
                     x = range(self.n_trn_flds, self.n_trn_flds*len(self.loss_vals[1])+1, self.n_trn_flds)
                     y = self.metric_vals[self.main_metric_idx]
                     self.metric_ax.plot(x, y, color=palette[1])
-                    best = np.min(y) if self.metric_cbs[self.main_metric_idx].lower_metric_better else np.max(y)
+                    best = np.nanmin(y) if self.metric_cbs[self.main_metric_idx].lower_metric_better else np.nanmax(y)
                     self.metric_ax.plot([1,x[-1]], [best,best], label=f'Best = {best:.3E}', linestyle='--', color=palette[2])
                     self.metric_ax.legend(loc='upper left', fontsize=0.8*self.plot_settings.leg_sz)
                     self.metric_ax.grid(True, which="both")
@@ -378,14 +379,15 @@ class MetricLogger(Callback):
         
         if save_best:
             if self.main_metric_idx is None or not self.lock_to_metric or len(losses) > 1:  # Tracking SWA only supported for loss
-                idx = np.unravel_index(np.argmin(losses), losses.shape)[-1]
-                results['loss'] = np.min(losses)
+                idx = np.unravel_index(np.nanargmin(losses), losses.shape)[-1]
+                results['loss'] = np.nanmin(losses)
             else:
-                idx = np.argmin(self.metric_vals[self.main_metric_idx]) if self.metric_cbs[self.main_metric_idx].lower_metric_better else \
-                    np.argmax(self.metric_vals[self.main_metric_idx])
+                idx = np.nanargmin(self.metric_vals[self.main_metric_idx]) if self.metric_cbs[self.main_metric_idx].lower_metric_better else \
+                    np.nanargmax(self.metric_vals[self.main_metric_idx])
                 results['loss'] = losses[0][idx]
         else:
-            results['loss'] = np.min(losses[:,-1:])
+            results['loss'] = np.nanmin(losses[:,-1:])
             idx = -1
-        for c,v in zip(self.metric_cbs,metrics[:,idx]): results[c.name] = v
+        if len(self.metric_cbs) > 0:
+            for c,v in zip(self.metric_cbs,metrics[:,idx]): results[c.name] = v
         return results
