@@ -26,7 +26,7 @@ from ..interpretation.features import get_nn_feat_importance
 from ..metrics.eval_metric import EvalMetric
 from ...plotting.plot_settings import PlotSettings
 from ...utils.statistics import uncert_round
-from ...utils.misc import to_np, to_device
+from ...utils.misc import to_np, to_device, is_partially
 
 __all__ = ['Model']
 
@@ -128,18 +128,17 @@ class Model(AbsModel):
             else:                                             self.fit_params.loss_func.weight  = self.fit_params.w
             self.fit_params.loss_val = self.fit_params.loss_func(self.fit_params.y_pred, self.fit_params.y)
         for c in self.fit_params.cbs: c.on_forwards_end()
-        if self.fit_params.state != 'train': return
-
-        self.fit_params.opt.zero_grad()
-        for c in self.fit_params.cbs: c.on_backwards_begin()
-        self.fit_params.loss_val.backward()
-        for c in self.fit_params.cbs: c.on_backwards_end()
-        self.fit_params.opt.step()
+        if self.fit_params.state == 'train':
+            self.fit_params.opt.zero_grad()
+            for c in self.fit_params.cbs: c.on_backwards_begin()
+            self.fit_params.loss_val.backward()
+            for c in self.fit_params.cbs: c.on_backwards_end()
+            self.fit_params.opt.step()
         for c in self.fit_params.cbs: c.on_batch_end()
 
     def fit(self, n_epochs:int, fy:FoldYielder, bs:int, bulk_move:bool=True, train_on_weights:bool=True, trn_idxs:Optional[List[int]]=None,
             val_idx:Optional[int]=None,  cbs:Optional[Union[AbsCallback,List[AbsCallback]]]=None, cb_savepath:Path=Path('train_weights'),
-            model_bar:Optional[master_bar]=None) -> List[AbsCallback]:
+            model_bar:Optional[master_bar]=None, visible_bar:bool=True) -> List[AbsCallback]:
         r'''
         Fit network to training data according to the model's loss and optimiser.
         
@@ -174,15 +173,14 @@ class Model(AbsModel):
 
         self.fit_params = FitParams(cbs=cbs, cyclic_cbs=cyclic_cbs, loss_cbs=loss_cbs, metric_log=metric_log, stop=False, n_epochs=n_epochs, fy=fy,
                                     val_idx=val_idx, bs=bs, bulk_move=bulk_move, train_on_weights=train_on_weights, cb_savepath=Path(cb_savepath),
-                                    loss_func=self.loss, opt=self.opt)
+                                    loss_func=self.loss, opt=self.opt, val_requires_grad=False)
         self.fit_params.cb_savepath.mkdir(parents=True, exist_ok=True)
-        if inspect.isclass(self.fit_params.loss_func) or isinstance(self.fit_params.loss_func, partial): self.fit_params.loss_func = self.fit_params.loss_func()
+        if inspect.isclass(self.fit_params.loss_func) or is_partially(self.fit_params.loss_func): self.fit_params.loss_func = self.fit_params.loss_func()
         self.fit_params.partial_by = partialler(BatchYielder, objective=self.objective, use_weights=self.fit_params.train_on_weights,
                                                 bulk_move=self.fit_params.bulk_move, input_mask=self.input_mask)
 
         if trn_idxs is None: trn_idxs = list(range(fy.n_folds))
         if val_idx is not None and val_idx in trn_idxs: trn_idxs.remove(val_idx)
-        shuffle(trn_idxs)
         self.fit_params.trn_idxs,self.fit_params.val_idx = trn_idxs,val_idx
         if self.fit_params.val_idx is not None:
             if bulk_move:
@@ -197,6 +195,8 @@ class Model(AbsModel):
             self.model.train()
             self.fit_params.state = 'train'
             self.fit_params.epoch += 1
+            shuffle(trn_idxs)
+            self.fit_params.trn_idxs = trn_idxs
             for c in self.fit_params.cbs: c.on_epoch_begin()
             for self.fit_params.trn_idx in self.fit_params.trn_idxs:
                 self.fit_params.sub_epoch += 1
@@ -209,40 +209,45 @@ class Model(AbsModel):
 
             if self.fit_params.val_idx is not None:
                 self.model.eval()
-                self.fit_params.state = 'valid'
-                for c in self.fit_params.cbs: c.on_epoch_begin()
-                self.fit_params.by = val_by if bulk_move else val_by(**self.fit_params.fy.get_fold(self.fit_params.val_idx))
-                for c in self.fit_params.cbs: c.on_fold_begin()
-                for b in self.fit_params.by: self._fit_batch(*b)
-                for c in self.fit_params.cbs: c.on_fold_end()
-                for c in self.fit_params.cbs: c.on_epoch_end()
-            del self.fit_params.by
+                with torch.set_grad_enabled(self.fit_params.val_requires_grad):
+                    self.fit_params.state = 'valid'
+                    for c in self.fit_params.cbs: c.on_epoch_begin()
+                    self.fit_params.by = val_by if bulk_move else val_by(**self.fit_params.fy.get_fold(self.fit_params.val_idx))
+                    for c in self.fit_params.cbs: c.on_fold_begin()
+                    for b in self.fit_params.by: self._fit_batch(*b)
+                    for c in self.fit_params.cbs: c.on_fold_end()
+                    for c in self.fit_params.cbs: c.on_epoch_end()
 
         try:
             for c in self.fit_params.cbs: c.set_model(self)
-            for c in self.fit_params.cbs: c.on_train_begin()
-            for e in progress_bar(range(self.fit_params.n_epochs), parent=model_bar):
+            for c in self.fit_params.cbs:
+                c.on_train_begin()
+            for e in progress_bar(range(self.fit_params.n_epochs), parent=model_bar, display=visible_bar):
                 fit_epoch()
                 if self.fit_params.stop: break
             if self.fit_params.val_idx is not None: del val_by
             for c in self.fit_params.cbs: c.on_train_end()
         finally:
             self.fit_params = None
+            torch.cuda.empty_cache()
         return cbs
 
     def _predict_by(self, by:BatchYielder, pred_cb:PredHandler=PredHandler(), cbs:Optional[Union[AbsCallback,List[AbsCallback]]]=None) -> np.ndarray:
         if cbs is None: cbs = []
         elif not is_listy(cbs): cbs = [cbs]
         cbs.append(pred_cb)
-        self.fit_params = FitParams(cbs=cbs, by=by, state='test')
+        self.fit_params = FitParams(cbs=cbs, by=by, state='test', val_requires_grad=False)
         try:
             for c in self.fit_params.cbs: c.set_model(self)
             self.model.eval()
-            for c in self.fit_params.cbs: c.on_pred_begin()
-            for b in self.fit_params.by: self._fit_batch(*b)
-            for c in self.fit_params.cbs: c.on_pred_end()
+            with torch.set_grad_enabled(self.fit_params.val_requires_grad):
+                for c in self.fit_params.cbs: c.on_pred_begin()
+                for b in self.fit_params.by: self._fit_batch(*b)
+                for c in self.fit_params.cbs: c.on_pred_end()
         finally:
             self.fit_params = None
+            cbs.pop()  # Remove pred_cb to avoid mutating cbs
+            torch.cuda.empty_cache()
         return pred_cb.get_preds()
 
     def _predict_array(self, inputs:Union[np.ndarray,pd.DataFrame,Tensor,Tuple], as_np:bool=True, pred_cb:PredHandler=PredHandler(),
@@ -278,7 +283,7 @@ class Model(AbsModel):
                                                                        objective=self.objective, shuffle=False, bulk_move=bs is None,
                                                                        input_mask=self.input_mask, drop_last=False)
         self.fit_params = FitParams(cbs=[], by=inputs, state='valid', loss_func=self.loss)
-        if inspect.isclass(self.fit_params.loss_func) or isinstance(self.fit_params.loss_func, partial): self.fit_params.loss_func = self.fit_params.loss_func()
+        if inspect.isclass(self.fit_params.loss_func) or is_partially(self.fit_params.loss_func): self.fit_params.loss_func = self.fit_params.loss_func()
         self.model.eval()
         loss,cnt = 0,0
         try:
@@ -293,13 +298,12 @@ class Model(AbsModel):
 
     def _predict_folds(self, fy:FoldYielder, pred_name:str='pred', pred_cb:PredHandler=PredHandler(), cbs:Optional[List[AbsCallback]]=None,
                        bs:Optional[int]=None) -> None:
-        pred_call = partialler(self._predict_array, pred_cb=pred_cb, cbs=cbs, bs=bs)
         for fold_idx in progress_bar(range(len(fy))):
             if not fy.test_time_aug:
-                pred = pred_call(fy.get_fold(fold_idx)['inputs'])
+                pred = self._predict_array(fy.get_fold(fold_idx)['inputs'], pred_cb=pred_cb, cbs=cbs, bs=bs)
             else:
                 tmpPred = []
-                for aug in range(fy.aug_mult): tmpPred.append(pred_call(fy.get_test_fold(fold_idx, aug)['inputs']))
+                for aug in range(fy.aug_mult): tmpPred.append(self._predict_array(fy.get_test_fold(fold_idx, aug)['inputs'], pred_cb=pred_cb, cbs=cbs, bs=bs))
                 pred = np.mean(tmpPred, axis=0)
 
             if self.n_out > 1: fy.save_fold_pred(pred, fold_idx, pred_name=pred_name)
@@ -375,8 +379,8 @@ class Model(AbsModel):
             momentum/beta_1 of optimiser
         '''
         
-        if   'betas'    in self.opt.param_groups: return self.opt.param_groups[0]['betas'][0]
-        elif 'momentum' in self.opt.param_groups: return self.opt.param_groups[0]['momentum']
+        if   'betas'    in self.opt.param_groups[0]: return self.opt.param_groups[0]['betas'][0]
+        elif 'momentum' in self.opt.param_groups[0]: return self.opt.param_groups[0]['momentum']
 
     def set_mom(self, mom:float) -> None:
         r'''
@@ -386,8 +390,8 @@ class Model(AbsModel):
             mom: momentum/beta_1 of optimiser
         '''
 
-        if   'betas'    in self.opt.param_groups: self.opt.param_groups[0]['betas'][0] = mom
-        elif 'momentum' in self.opt.param_groups: self.opt.param_groups[0]['momentum'] = mom
+        if   'betas'    in self.opt.param_groups[0]: self.opt.param_groups[0]['betas'] = (mom, self.opt.param_groups[0]['betas'][1])
+        elif 'momentum' in self.opt.param_groups[0]: self.opt.param_groups[0]['momentum'] = mom
     
     def save(self, name:str) -> None:
         r'''
@@ -480,6 +484,20 @@ class Model(AbsModel):
 
         return self.tail.get_out_size()
 
+    def freeze_layers(self) -> None:
+        r'''
+        Make parameters untrainable
+        '''
+
+        for p in self.model.parameters(): p.requires_grad = False
+    
+    def unfreeze_layers(self) -> None:
+        r'''
+        Make parameters trainable
+        '''
+
+        for p in self.model.parameters(): p.requires_grad = True
+
 
 class OldModel(Model):
     r'''
