@@ -25,8 +25,9 @@ __all__ = ['train_models']
 
 
 def train_models(fy:FoldYielder, n_models:int, bs:int, model_builder:ModelBuilder, n_epochs:int, patience:Optional[int]=None, loss_is_meaned:bool=True,
-                 cb_partials:Optional[List[Callable[[],Callback]]]=None, metric_partials:Optional[List[Callable[[],EvalMetric]]]=None,
-                 pred_cb:Callable[[],PredHandler]=PredHandler, train_on_weights:bool=True, bulk_move:bool=True, start_mode_id:int=0,
+                 cb_partials:Optional[List[Callable[[],Callback]]]=None, metric_partials:Optional[List[Callable[[],EvalMetric]]]=None, save_best:bool=True,
+                 pred_cb:Callable[[],PredHandler]=PredHandler, train_on_weights:bool=True, bulk_move:bool=True, start_model_id:int=0,
+                 excl_idxs:Optional[List[int]]=None, unique_trn_idxs:bool=False,
                  live_fdbk:bool=IN_NOTEBOOK, live_fdbk_first_only:bool=False, live_fdbk_extra:bool=True, live_fdbk_extra_first_only:bool=False,
                  savepath:Path=Path('train_weights'), plot_settings:PlotSettings=PlotSettings()) \
         -> Tuple[List[Dict[str,float]],List[Dict[str,List[float]]],List[Dict[str,float]]]:
@@ -59,12 +60,21 @@ def train_models(fy:FoldYielder, n_models:int, bs:int, model_builder:ModelBuilde
             :class:`~lumin.nn.callbacks.monitors.SaveBest` and :class:`~lumin.nn.callbacks.monitors.EarlyStopping` will also act on the (first) metric set to 
             `main_metric` instead of loss, except when another callback produces an alternative loss and model
             (like :class:`~lumin.nn.callbacks.model_callbacks.SWA`).
+        save_best: if true, will save the best performing model as the final model, otherwise will save the model state as per the end of training.
+            A copy of the best model will still be saved anyway.
         pred_cb: pred_cb: :class:`~lumin.nn.callbacks.pred_handlers.PredHandler` callback to determin how predictions are computed.
             Default simply returns the model predictions. Other uses could be e.g. running argmax on a multiclass classifier
         train_on_weights: If weights are present in training data, whether to pass them to the loss function during training
         bulk_move: if true, will optimise for speed by using more RAM and VRAM
-        start_mode_id: model ID at whcih to start training,
+        start_model_id: model ID at whcih to start training,
             i.e. if training was interupted, this can be set to resume training form the last model which was trained
+        excl_idxs: optional list of fold indeces to exclude from training and validation
+        unique_trn_idxs: if false, then fold indeces can be shared, 
+            e.g. if `fy` contains 10 folds and five models are requested, each model will be trained on 9 folds.
+            if true, each model will every model will be trained on different folds,
+            e.g. if `fy` contains 10 folds and five models are requested, each model will be trained on 2 folds and no same fold is used to train more than one model
+            This is useful when the amount of training data exceeds the amount required to train a single model:
+            it can be split into a large number of folds and a set of decorellated models trained.
         live_fdbk: whether or not to show any live feedback at all during training (slightly slows down training, but helps spot problems)
         live_fdbk_first_only: whether to only show live feedback for the first model trained (trade off between time and problem spotting)
         live_fdbk_extra: whether to show extra information live feedback (further slows training)
@@ -81,15 +91,28 @@ def train_models(fy:FoldYielder, n_models:int, bs:int, model_builder:ModelBuilde
     results,histories,cycle_losses,savepath = [],[],[],Path(savepath)
     if cb_partials is None: cb_partials = []
     if not is_listy(cb_partials): cb_partials = [cb_partials]
+    if metric_partials is None: metric_partials = []
+    if not is_listy(metric_partials): metric_partials = [metric_partials]
+    if excl_idxs is None: excl_idxs = []
+    if not is_listy(excl_idxs): excl_idxs = [excl_idxs]
+    idxs = [i for i in range(fy.n_folds) if i not in excl_idxs]
+    trn_idx_sets,val_idxs = [],[]
+    if unique_trn_idxs:
+        n = len(idxs)//n_models
+        if n == 0: raise ValueError(f"{len(idxs)} training folds are not enough to train {n_models} with unique data.")
+        for i in range(n_models):
+            val_idxs.append(idxs[i])
+            trn_idx_sets.append([j for j in idxs if j != idxs[i] and j not in np.array(trn_idx_sets).flatten()][:n])
 
-    model_rng = range(start_mode_id, n_models)
+    model_rng = range(start_model_id, n_models)
     for i in model_rng: os.system(f"rm -r {savepath}/model_id_{i}")
     model_bar = master_bar(model_rng) if IN_NOTEBOOK else progress_bar(model_rng)
     train_tmr = timeit.default_timer()
     for model_num in (model_bar):    
         if IN_NOTEBOOK: model_bar.show()
-        val_idx = model_num % fy.n_folds
-        print(f"Training model {model_num+1} / {n_models}, Val ID = {val_idx}")
+        val_idx  = val_idxs[model_num]     if unique_trn_idxs else idxs[model_num % len(idxs)]
+        trn_idxs = trn_idx_sets[model_num] if unique_trn_idxs else [j for j in idxs if j != val_idx]
+        print(f"Training model {model_num+1} / {n_models}, Valid Index = {val_idx}, Train indices= {trn_idxs}")
         if model_num == 1:
             if live_fdbk_first_only: live_fdbk = False  # Only show fdbk for first training
             elif live_fdbk_extra_first_only: live_fdbk_extra = False  # Only show full fdbk info for first training
@@ -102,12 +125,13 @@ def train_models(fy:FoldYielder, n_models:int, bs:int, model_builder:ModelBuilde
         for c in cb_partials: cbs.append(c())
         for c in metric_partials: cbs.append(c())
         metric_log = MetricLogger(show_plots=live_fdbk, extra_detail=live_fdbk_extra, loss_is_meaned=loss_is_meaned)
-        cbs += [metric_log, SaveBest(auto_reload=True, loss_is_meaned=loss_is_meaned)]
+        cbs += [metric_log, SaveBest(auto_reload=save_best, loss_is_meaned=loss_is_meaned)]
         if patience is not None: cbs.append(EarlyStopping(patience=patience, loss_is_meaned=loss_is_meaned))
         for c in cbs: c.set_plot_settings(plot_settings)
 
         model_tmr = timeit.default_timer()
-        model.fit(n_epochs=n_epochs, fy=fy, bs=bs, bulk_move=bulk_move, train_on_weights=train_on_weights, val_idx=val_idx, cbs=cbs, cb_savepath=model_dir)
+        model.fit(n_epochs=n_epochs, fy=fy, bs=bs, bulk_move=bulk_move, train_on_weights=train_on_weights, trn_idxs=trn_idxs, val_idx=val_idx,
+                  cbs=cbs, cb_savepath=model_dir)
         print(f"Model took {timeit.default_timer()-model_tmr:.3f}s\n")
         model.save(model_dir/f'train_{model_num}.h5')
 
@@ -115,7 +139,7 @@ def train_models(fy:FoldYielder, n_models:int, bs:int, model_builder:ModelBuilde
         cycle_losses.append([])
         for c in cbs:
             if hasattr(c, 'cycle_save') and c.cycle_save: cycle_losses[-1] = c.cycle_losses
-        results.append(metric_log.get_results(save_best=True))
+        results.append(metric_log.get_results(save_best=save_best))
         print(f"Scores are: {results[-1]}")
         results[-1]['path'] = model_dir
         with open(savepath/'results_file.pkl', 'wb') as fout: pickle.dump(results, fout)
